import pygame

from pykitten.kittens import Kitten

from globs import GLOBALS

class Camera(Kitten):
    def __init__(self, focus_on):
        img = pygame.Surface((1,1))
        Kitten.__init__(self, img)
        self.max_vel = 5
        self.focus_on = focus_on
        self.min_x = None
        self.min_y = None
        self.max_x = None
        self.max_y = None

    def set_max(self, x, y):
        self.max_x = x
        self.max_y = y
        self.min_x = 0
        self.min_y = 0

    def add_vel(self, vel):
        add_x, add_y = vel
        vx, vy = self.vel

        if vx + add_x < self.max_vel:
            vx += add_x
        if vy + add_y < self.max_vel:
            vy += add_y

        self.set_vel((vx,vy))

    def update(self):
        sx, sy = GLOBALS["screen_size"]
        x, y = self.focus_on.rect.center
        nx = x - sx / 2
        ny = y - sy / 2 - 100
        dx, dy = (nx - self.pos[0], ny - self.pos[1])
        if abs(dx) > 20 or abs(dy) > 20:
            #move to focus position if camera is too off
            self.set_pos((nx,ny))
        else:
            #follow vel
            self.set_vel(self.focus_on.vel)
        Kitten.update(self)
        self.check_max()

    def check_max(self):
        if self.max_x is None:
            return
        x, y = self.pos
        if x >  self.max_x:
            x = self.max_x
        elif x < self.min_x:
            x = self.min_x

        if y > self.max_y:
            y = self.max_y
        elif y < self.min_y:
            y = self.min_y

        self.set_pos((x,y))
