import pygame

from pykitten import utils
from pykitten.data import *
from pykitten.gui import TextLine



GLOBALS = {}
GLOBALS["jb"] = JukeBox(["bgm.xm"])
GLOBALS["screen_size"] = (640, 480)
GLOBALS["tile_size"] = 32

GLOBALS["clan_levels"] = {
    "wood": "ki.map",
    "day": "hi.map",
    "fire": "ka.map",
    "life": "sei.map",
    "earth": "ji.map",
    "night": "yoru.map",
    "metal": "kin.map",
    "water": "mizu.map",
    "death": "shi.map"
}

#note: change init_player_data() if you want to change the initial values
GLOBALS["player_data"] = {}

GLOBALS["clan_colors"] = {
    "wood": (145,255,145),
    "day": (255,255,155),
    "fire": (255,81,81),
    #"life": (235,235,235),
    "life": (255,255,255),
    "earth": (148,91,35),
    "night": (0,64,128),
    "metal": (180,185,205),
    "water": (136,255,222),
    "death": (62,62,62)
}

def load_nanji():
    GLOBALS["nanji"] = load_ninjas(load_image("nanji.png"))

def load_ninjas(main_img):
    images = {}

    #walking
    size = (50,41)
    coords = [(5,75),(61,75),(117,75),(173,75),(229,75)]
    images["walking"] = load_subsurf(main_img, size, coords)
    images["walking"].append(images["walking"][3])
    images["walking"].append(images["walking"][2])
    images["walking"].append(images["walking"][1])

    #idle
    size = (34,47)
    coords = [(5,138),(45,138),(85,138)]
    images["idle"] = load_subsurf(main_img, size, coords)

    #sprinting
    size = (71, 36)
    coords = [(5,17), (82,17), (159,17), (236,17), (313,17)]
    images["sprinting"] = load_subsurf(main_img, size, coords)
    images["sprinting"].append(images["sprinting"][3])
    images["sprinting"].append(images["sprinting"][2])
    images["sprinting"].append(images["sprinting"][1])

    #jump start
    sizes = [(34,47), (34,47),(41,54),(47,64)]
    coords = [ (5,207), (46,207), (87,207), (134,207) ]
    images["jump"] = load_subsurfs(main_img, sizes, coords)

    #in flight
    sizes = [ (71,35), (63,47) ]
    coords = [ (187,207), (264,207) ]
    images["flight"] = load_subsurfs(main_img, sizes, coords)

    #land
    sizes = [(58,43), (56,35), (34,47) ]
    coords = [( 333,207), (397,207), (459,207)]
    images["landing"] = load_subsurfs(main_img, sizes, coords)

    #throwing
    size = (34,47)
    coords = [ (160,138), (200,138), (240,138), (280,138) ]
    images["throwing"] = load_subsurf(main_img, size, coords)

    #thrown shuriken
    size = (36, 24)
    coords = [ (370,84), (412,84), (370,113), (412,113) ]
    images["shuri_throw"] = load_subsurf(main_img, size, coords)
    images["shuri_throw"] = add_empty(images["shuri_throw"], size, 0)

    #flying shuriken
    size = (26,11)
    coords = [ (370,160), (403,160) ]
    images["shuri_fly"] = load_subsurf(main_img, size, coords)

    #ducking
    size = 64,29
    coords = [(417,24)]
    images["ducking"] = load_subsurf(main_img, size, coords)

    #mirrored versions of everything
    for k in images.keys():
        imglist = []
        for img in images[k]:
            imglist.append(pygame.transform.flip(img, True, False))
        images[k + "_l"] = imglist

    return images

def load_ninjats():
    #colorized nanjis
    clans = GLOBALS["clan_colors"]
    nanji = GLOBALS["nanji"]
    images = {}

    for c in clans:
        imgs = {}
        for k in nanji.keys():
            imglist = []
            for i in nanji[k]:
                imglist.append(colorize(i,clans[c]))
            imgs[k] = imglist
        images[c] = imgs
    GLOBALS["ninjats"] = images

def load_backgrounds():
    filenames = [
        "fuji.png",
    ]

    GLOBALS["backgrounds"] = load_filenames(filenames)
    GLOBALS["backgrounds"]["gameboard.png"] = load_image("gameboard.png")
    GLOBALS["backgrounds"]["main_menu.png"] = load_image("main_menu.png")
    GLOBALS["backgrounds"]["credits.png"] = load_image("credits.png")
    GLOBALS["backgrounds"]["game_over.png"] = load_image("game_over.png")
    GLOBALS["backgrounds"]["the_end.png"] = load_image("the_end.png")


def load_tilesets():
    filenames = [
        "tileset",
    ]

    for name in filenames:
        filename = name + ".png"
        ts_id = "ts_" + name
        tlist = load_grid(filename, 32, 32)
        tdict = {}
        for row in xrange(len(tlist)):
            for col in xrange(len(tlist[0])):
                tdict[col,row] = tlist[row][col]
        GLOBALS[ts_id] = tdict

def load_objects():
    #load special map object-images for editor
    #(i.e. start and exit positions, dialog triggers)

    obj = {  #TODO: add more objects + a way to define their image
        "start":None,
        "exit":None,
        "enemy":None,
        "dojo":"dojo.png",
        "dtrigger":None,
    }

    objects = {}
    surf = pygame.Surface((GLOBALS["tile_size"], GLOBALS["tile_size"]))
    for o in obj.keys():
        if obj[o] is None: #no image
            surf.set_colorkey(None)
            surf.fill((255,0,255))
            surf.set_colorkey((255,0,255))
            img = surf.copy()
            txt = TextLine(o, fontsize=10, bgcolor=(0,0,0))
            img.blit(txt, utils.center(txt.get_size(), img.get_size()))
        else:
            img = load_image(obj[o])
        objects[o] = img

    #dojo entry
    objects["dojo_entry"] = load_image("dojo_entry.png", alpha=True)
    GLOBALS["objects"] = objects

def load_portraits():
    images = {}

    #so far we have only Nanji
    size = (19,26)
    coords = [(309,84)]
    main_img = load_image("nanji.png")
    images["nanji"] = scale2x(load_subsurf(main_img, size, coords)[0], 4)
    GLOBALS["portraits"] = images

def load_clanstuff():
    #signets
    images = {}
    main_img = load_image("signets.png")

    #large, medium, tiny
    sizes = [(72,72), (28,28), (14,14)]
    coords = [(12,17), (23,94), (65,101)]
    images["wood"] = load_subsurfs(main_img, sizes, coords)

    coords = [(112,17), (121,93),(163,100)]
    images["day"] = load_subsurfs(main_img, sizes, coords)

    coords = [(208,17), (214,94), (256,101) ]
    images["fire"] = load_subsurfs(main_img, sizes, coords)

    coords = [(309,17), (316,92), (358,99)]
    images["life"] = load_subsurfs(main_img, sizes, coords)

    coords = [(413,17), (420,92), (462,99)]
    images["earth"] = load_subsurfs(main_img, sizes, coords)

    coords = [(17,151), (25,228), (67,235)]
    images["night"] = load_subsurfs(main_img, sizes, coords)

    coords = [(146,150),(153,225),(195,232)]
    images["metal"] = load_subsurfs(main_img, sizes, coords)

    coords = [(254,149),(261,227), (303,234)]
    images["water"] = load_subsurfs(main_img, sizes, coords)

    coords = [(378,151),(386,227), (428,234)]
    images["death"] = load_subsurfs(main_img, sizes, coords)

    GLOBALS["signets"] = images

    #kanji tooltip
    kanji = {
        "wood": ((11,11),(78,2)),
        "day": ((8,11),(177,2)),
        "fire": ((11,11),(274,1)),
        "life": ((11,11),(376,1)),
        "earth": ((11,11),(486,1)),
        "night": ((11,11),(100,133)),
        "metal": ((11,11),(222,133)),
        "water": ((11,11),(342,133)),
        "death": ((11,11),(455,133))
    }

    GLOBALS["kanji"] = load_dict_subsurfs(main_img, kanji)
    GLOBALS["kanji"]["clan"] = load_image("clan.png")

def load_uistuff():
    filenames = [
        "board_empty.png", #these.. don't belong here.. whatever.
        "board_filled.png",
        "door_both.png",
        "door_left.png",
        "door_right.png",
        "mysterious.png",
        "vortex.png",
        "logo.png"
    ]

    images = load_filenames(filenames, scale=False, colorkey=(255,0,255))

    #heart
    size = (15,15)
    coords = [(302,115),(321,115)]
    main_image = load_image("nanji.png")
    imglist = load_subsurf(main_image, size, coords)
    images["heart"] = imglist[0]
    images["heart_empty"] = imglist[1]

    #life (nanji portrait)
    life = pygame.Surface((15,15))
    life.fill((80,80,80))
    images["life"] = load_subsurf(load_image("nanji.png"), (19,26), [(309,84)])[0]

    #green light for game board
    images["green_light"] = load_image("greenlight.png", colorkey=(255,0,255))
    images["donotpress"] = load_image("donotpress.png")

    GLOBALS["ui_stuff"] = images

def load_sounds():
    filenames = get_filenames(path=filepath("sfx"), suffix=".wav")
    GLOBALS["sfx"] = load_sfx(filenames)
    for k in GLOBALS["sfx"].keys():
        GLOBALS["sfx"][k].set_volume(0.3)

#small utility functions for data loading stuff
def load_dict_subsurfs(main_img, dict):
    images = {}
    for k in dict.keys():
        size = dict[k][0]
        coord = dict[k][1]
        img = pygame.Surface(size)
        img.set_colorkey((255,0,255))
        img.blit(main_img, (0,0), [coord, size])
        images[k] = img
    return images

def load_subsurf(main_img, size, coords):
    images = []
    for c in coords:
        img = pygame.Surface(size)
        img.set_colorkey((255,0,255))
        img.blit(main_img, (0,0), [c, size])
        #img = scale2x(img)
        images.append(img)
    return images

def load_subsurfs(main_img, sizes, coords):
    images = []
    for i in xrange(len(sizes)):
        img = pygame.Surface(sizes[i])
        img.set_colorkey((255,0,255))
        img.blit(main_img, (0,0), [coords[i], sizes[i]])
        #img = scale2x(img)
        images.append(img)
    return images

def blit_lists(images1, images2, pos):
    images = []
    for i in xrange(len(images1)):
        img = images1[i].copy()
        img.blit(images2[i], pos)
        images.append(img)
    return images

def scale2x(img, s=2):
    w, h = img.get_size()
    return pygame.transform.scale(img, (w*s, h*s))

def load_filenames(filenames, scale=True, colorkey=None):
    images = {}
    for name in filenames:
        img = load_image(name, colorkey=colorkey)
        if scale:
            img = scale2x(img)
        images[name] = img
    return images

def add_empty(imagelist, size, index):
    img = pygame.Surface(size)
    img.set_alpha(0)
    imagelist.insert(index, img)
    return imagelist

def colorize(img, color):  #ninjat coloring
    img = img.copy()
    surf = pygame.Surface(img.get_size())
    surf.fill(color)
    surf.set_alpha(180)
    img.blit(surf, (0,0))
    img.set_colorkey(img.get_at((0,0)))
    img.set_alpha(None)
    return img


def init_player_data(lives=9, health=9, unlocked=[]):
    GLOBALS["player_data"]["lives"] = lives
    GLOBALS["player_data"]["health"] = health
    GLOBALS["player_data"]["unlocked"] = unlocked


#call this to set all variables
def set_globals():
    load_nanji()
    load_ninjats()
    load_tilesets()
    load_backgrounds()
    load_objects()
    load_portraits()
    load_clanstuff()
    load_uistuff()
    load_sounds()

    init_player_data()

