import pygame
from pygame.locals import *

from pykitten.utils import Timer
from pykitten.engine import State
from pykitten.kittens import Kitten
from pykitten.gui import StaticWidget, TextLine

from camera import Camera
from globs import GLOBALS
from nanji import Nanji, Ninjat
from background import Background
from bullets import Shuriken
from map import Map
from showdialog import ShowDialog, PauseState
from gameover import GameOver
from fadestate import FadeState

tsize = GLOBALS["tile_size"]
player_data = GLOBALS["player_data"]
sfx_hit = "hit.wav"
#sfx_shurikenhit = "shurikenhit.wav"

class Level(State):
    def __init__(self, lvl_map="tutorial.map", testing=False, prev=None):
        State.__init__(self)
        self.fps = 60
        self.prev = prev
        tsize = GLOBALS["tile_size"]
        self.trigger = None
        self.exit_timer = Timer(50)
        self.exit_timer.deactivate()
        pygame.mixer.music.set_volume(1)
        GLOBALS["jb"].play("bgm.xm", -1)

        player_data["health"] = 9  #always full health on map start
        #player_data["lives"] = #for gameover testing

        #scrolling background - maybe later XD
        #bg_back = GLOBALS["backgrounds"]["fuji.png"]
        #bg_front = GLOBALS["backgrounds"]["trees.png"]
        #self.scroll_bg = Background(bg_back, bg_front)

        #non-scrolling background
        self.bg = GLOBALS["backgrounds"]["fuji.png"]

        #map
        self.map_filename = lvl_map
        self.map = Map(lvl_map)
        self.lvl_map = self.map["tilemap"]
        x, y = self.map["max"]
        self.lvl_img, self.blocks = self.map.draw(editing=False)
        self.lvl_layer = pygame.Surface(self.lvl_img.get_size())

        #other spritegroups
        self.bullets = pygame.sprite.Group()

        #player
        self.player = Nanji(None)
        self.player.set_pos((self.map["start"][0]*tsize, self.map["start"][1]*tsize))
        self.sprites.add(self.player)

        #camera
        self.camera = Camera(self.player)
        screen_x, screen_y = GLOBALS["screen_size"]
        mx = self.map["max"][0] * tsize - screen_x
        my = self.map["max"][1] * tsize - screen_y + 4*tsize
        self.camera.set_max(mx,my)
        self.player.camera = self.camera
        self.camera.update()

        #add enemies
        self.enemies = pygame.sprite.Group()
        clan = self.map["options"]["clan"]
        for e in self.map.enemies:
            ninjat = Ninjat(self.camera, clan)
            ninjat.set_pos((e[0]*tsize, e[1]*tsize-32))#FIXME:enemies are spawned too low..?
            self.enemies.add(ninjat)
            self.sprites.add(ninjat)

        #ui layer
        self.ui_layer = StaticWidget((GLOBALS["screen_size"][0], 30))
        self.ui_layer.fill((255,0,255))
        self.ui_layer.set_colorkey((255,0,255))
        self.ui_layer.set_pos((0,0))
        self.heart_img = GLOBALS["ui_stuff"]["heart"]
        self.heart_empty_img = GLOBALS["ui_stuff"]["heart_empty"]
        self.life_img = GLOBALS["ui_stuff"]["life"]
        self.fontholder = TextLine("", fontsize=16)
        self.update_ui()

        #keyboard events
        self.key_pressed = {
            "up": 0, "down": 0, "left": 0, "right": 0,
            "jump":0, "sprint":0, "throw": 0, "shuriken": 0,
        }

        self.add_kbevent(KEYDOWN, K_LEFT, self.set_keypress, k="left", v=1)  #move
        self.add_kbevent(KEYDOWN, K_RIGHT, self.set_keypress, k="right", v=1)
        self.add_kbevent(KEYDOWN, K_UP, self.set_keypress, k="up", v=1)
        self.add_kbevent(KEYDOWN, K_DOWN, self.set_keypress, k="down", v=1)
        self.add_kbevent(KEYDOWN, K_SPACE, self.set_keypress, k="jump", v=1)  #jump
        self.add_kbevent(KEYDOWN, K_LSHIFT, self.set_keypress, k="sprint", v=1)  #sprint
        self.add_kbevent(KEYDOWN, K_RSHIFT, self.set_keypress, k="sprint", v=1)
        self.add_kbevent(KEYDOWN, K_RCTRL, self.set_keypress, k="throw", v=1)  #throw
        self.add_kbevent(KEYDOWN, K_LCTRL, self.set_keypress, k="throw", v=1)

        self.add_kbevent(KEYUP, K_LEFT, self.set_keypress, k="left", v=0)
        self.add_kbevent(KEYUP, K_RIGHT, self.set_keypress, k="right", v=0)
        self.add_kbevent(KEYUP, K_UP, self.set_keypress, k="up", v=0)
        self.add_kbevent(KEYUP, K_DOWN, self.set_keypress, k="down", v=0)
        self.add_kbevent(KEYUP, K_SPACE, self.set_keypress, k="jump", v=0)
        self.add_kbevent(KEYUP, K_LSHIFT, self.set_keypress, k="sprint", v=0)
        self.add_kbevent(KEYUP, K_RSHIFT, self.set_keypress, k="sprint", v=0)
        self.add_kbevent(KEYUP, K_RCTRL, self.set_keypress, k="throw", v=0)
        self.add_kbevent(KEYUP, K_LCTRL, self.set_keypress, k="throw", v=0)

        if testing:
            self.add_kbevent(KEYUP, K_ESCAPE, self.quit)

    def main_start(self):
        pygame.mouse.set_visible(0)
        State.main_start(self)



    def reset_keys(self):
        self.key_pressed = {
            "up": 0, "down": 0, "left": 0, "right": 0,
            "jump":0, "sprint":0, "throw": 0, "shuriken": 0,
        }

    def update_other(self):
        if self.player.exiting:
            self.show_exit_anim()
            return

        kp = self.key_pressed
        vx = 0
        if kp["left"] and not kp["right"]:
            vx = -1
        if kp["right"] and not kp["left"]:
            vx = 1
        self.player.mod_vel(vx=vx)

        if kp["jump"]:
            self.player.jump()
        elif self.player.status==4:
            self.player.unjump()

        if kp["sprint"]:
            self.player.sprint()
            #self.scroll_bg.front_speed = 8
        elif self.player.status==2:
            self.player.unsprint()
            #self.scroll_bg.front_speed = 4

        if kp["down"]:
            self.player.duck()
        elif self.player.status==7:
            self.player.unduck()

        if kp["throw"]:
            kp["throw"] = 0
            if self.player.throw():
                self.bullets.add(Shuriken(self.player, self.camera))

    def set_keypress(self, k, v):
        self.key_pressed[k] = v

    def show_exit_anim(self):
        self.fps = 30
        self.reset_keys()
        if self.player.exiting == -1:
            self.player.set_vel((0,0))
            self.player.set_image(GLOBALS["nanji"]["idle"][0])
            if not self.exit_timer.active:
                self.exit_timer.activate()
        elif self.player.vel[1]:
            self.player.status = 5
        else:
            self.player.status = 1
            self.player.mod_vel(vx=1)

        if self.player.pos[0] > self.map["exit"][0]*tsize+10*tsize or \
            self.exit_timer.update():
            #TODO: play some funky tune / do SOMETHING on level exit
            if self.prev:
                self.prev.success = True
            self.quit()

    def show_dialog(self, dialog_id):
        self.pause()
        self.next = ShowDialog(dialog_id, self.screen)
        self.quit()

    def unpause(self):
        self.reset_keys()
        State.unpause(self)

    def check_collisions(self):
        #check bullet collisions
        b = self.bullets
        s = self.enemies
        bl = self.blocks

        #test if bullet hit player
        coll = pygame.sprite.spritecollide(self.player, self.bullets, False)
        for bullet in coll:
            if bullet.thrower is not self.player and bullet.flying and not \
                self.player.invincible:
                bullet.kill()
                self.player.get_hit()
                self.rem_health()

        #test if bullet hit wall
        dead = []
        for bullet in self.bullets:
            if bullet.rect.collidelist(bl) > -1:
                dead.append(bullet)
        for d in dead:
            if d.flying:
                d.kill()

        #test if bullet hit enemy (ninjats can't shoot each other)
        coll = pygame.sprite.groupcollide(s, b, False, False)
        for victim in coll.keys(): #my code gets weird...
            for murderer in coll[victim]:
                if murderer.thrower is self.player:
                    victim.die()
                    murderer.kill()

        #test player vs. enemy collisions
        coll = pygame.sprite.spritecollide(self.player, s, False)
        pbottom = self.player.rect.bottom
        for enemy in coll:
            if enemy.status == 8:  #enemy dead.. ignore!
                continue
            etop = enemy.rect.top
            if self.player.status==5 and pbottom >= etop:
                #enemy below player and player is falling -> enemy crushed!
                enemy.die()
                self.player.bounce()
            elif not self.player.invincible and not self.player.status==7:
                #not ducking -> get hit
                self.player.get_hit()
                self.rem_health()

    def rem_health(self):
        player_data["health"] -= 1
        GLOBALS["sfx"][sfx_hit].play()
        if player_data["health"] <= 0:
            self.player.die()
        self.update_ui()

    def update_ui(self):
        self.ui_layer.fill((255,0,255))

        #health
        x, y, add = 4, 2, self.heart_img.get_size()[0] + 2
        for i in xrange(9):
            if i < player_data["health"]:  #TODO: if i < hp
                self.ui_layer.blit(self.heart_img, (x,y))
            else:
                self.ui_layer.blit(self.heart_empty_img, (x,y))
            x += add

        #lives
        x, y, add = 595, 2, self.life_img.get_size()[0] + 2
        self.ui_layer.blit(self.life_img, (x,y))
        lives = player_data["lives"]
        life_text = self.fontholder.font.render("x"+str(lives), False, (255,255,255))
        self.ui_layer.blit(life_text, (x+add, y+10))

    def update_screen(self):
        for s in self.sprites:
            s.update_status()
            if not s.ai: #update player
                s.update()
                if not s.status==8 and not self.player.off_screen():
                    s.check_mapcollisions(self.blocks)
                    trig = s.check_trigcollisions(self.map.triggers)
                    if trig:
                        if trig.name=="exit":
                            self.player.exiting = 1
                        elif trig.name=="dtrigger":
                            self.show_dialog(trig.params)
                            if trig.params=="tut08":
                                self.player.exiting = -1
                                self.player.set_image(GLOBALS["nanji"]["idle"][0])
                else:
                        player_data["health"] = 9
                        player_data["lives"] -= 1
                        if player_data["lives"] > 0:
                            next = Level(self.map_filename, prev=self.prev)

                        else:
                            next = GameOver()
                        self.next = FadeState(self.screen, next.background, next)
                        self.quit()
            else:  #update ai
                if s.ai_update(self): #throw shuriken?
                    self.bullets.add(Shuriken(s, self.camera))

        self.bullets.update()
        self.camera.update()

        #draw bg
        #self.scroll_bg.update(self.player.vel[0])
        self.background.blit(self.bg, (0,0))

        #draw map and sprites
        self.lvl_layer.fill((255,0,255))
        self.lvl_layer.blit(self.lvl_img, self.camera.pos, [self.camera.pos, GLOBALS["screen_size"]])
        self.lvl_layer.set_colorkey((255,0,255))
        self.bullets.draw(self.lvl_layer)
        self.sprites.draw(self.lvl_layer)

        #pygame.draw.rect(self.lvl_layer, (255,255,0), self.player.colrect, 1) #draw collision rect of nanji

        if self.player.exiting:  #draw semi-black image over dojo entry while entering
            self.lvl_layer.blit(GLOBALS["objects"]["dojo_entry"], self.map.dojo_pos)

        self.background.blit(self.lvl_layer, (0,0), [self.camera.pos, GLOBALS["screen_size"]])
        self.ui_layer.draw(self.background)
        self.screen.blit(self.background, (0,0))
        pygame.display.flip()


