import pygame
from pygame.locals import *

from pykitten.data import save_object
from pykitten.utils import Timer
from pykitten.gui import ImgButton, TextLine, StaticWidget, ClickArea
from pykitten.engine import State

from showdialog import ShowDialog
from globs import GLOBALS
from level import Level
from outro import Outro


player_data = GLOBALS["player_data"]

class LevelSelection(State):
    def __init__(self, new=True):
        State.__init__(self)
        self.success = False
        self.new = new  #start tutorial before showing level selection!

        self.signet_pos = {
            "wood": (83,181),
            "day": (158,75),
            "fire": (284,34),
            "life": (410,75),
            "earth": (485,181),
            "night": (456,307),
            "metal": (348,368),
            "water": (220,368),
            "death": (112,307)
        }

        self.light_pos =  {
            "wood": (239,225),
            "day": (261,173),
            "fire": (315,155),
            "life": (369,173),
            "earth": (391,225),
            "night": (383,271),
            "metal": (345,307),
            "water": (285,307),
            "death": (247,271)
        }

        self.cur_blinking = None
        self.cur_blending = None
        self.cur_clan = None
        self.blink_timer = Timer(50)
        self.blink_count = 0
        self.blink_max = 10
        self.light = GLOBALS["ui_stuff"]["green_light"]

        r = pygame.Rect([281,203,78,77])
        r.inflate(-10, -10)
        bigredbutton = ClickArea(r, self.bigred_pressed)
        self.widgets.append(bigredbutton)
        self.donotpress = pygame.Rect([267,188,106,95])
        self.signets = GLOBALS["signets"]
        self.bg_image = GLOBALS["backgrounds"]["gameboard.png"].copy()
        self.ui_layer = pygame.Surface(GLOBALS["screen_size"])
        self.ui_layer.fill((255,0,255))
        self.ui_layer.set_colorkey((255,0,255))

        self.tooltip_layer = pygame.Surface(GLOBALS["screen_size"])
        self.tooltip_layer.fill((255,0,255))
        self.tooltip_layer.set_colorkey((255,0,255))
        self.tooltip = StaticWidget((80,20))
        self.dnp_tooltip = GLOBALS["ui_stuff"]["donotpress"]

        #level selection buttons
        self.signet_buttons = []
        levels = GLOBALS["clan_levels"]
        for k in self.signet_pos.keys():
            if self.signets.has_key(k) and levels.has_key(k):
                btn = ImgButton(self.signets[k][0], self.start_level, \
                    clan=k, level=levels[k])
                btn.set_pos(self.signet_pos[k])
                btn.rect = pygame.Rect([self.signet_pos[k],btn.get_size()])
                btn.rect.inflate(-20,-20)
                btn.clan = k
                self.widgets.append(btn)
                self.signet_buttons.append(btn)

        #show savegame progress
        self.show_signet(player_data["unlocked"])

        #keyboard events
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)


        self.background.blit(self.bg_image, (0,0))
        self.background.blit(self.tooltip_layer, (0,0))
        self.background.blit(self.ui_layer, (0,0))
        self.tooltip_layer.fill((255,0,255))

    def main_start(self):
        if pygame.mixer.music.get_busy():
            pygame.mixer.music.fadeout(300)

        pygame.mouse.set_visible(1)
        if self.new is True:
            self.new = "somewhere in between"
            self.cur_clan = None
            self.next = Level("tutorial.map", prev=self)
            self.pause()
            self.quit()
        elif self.new == "somewhere in between":
            self.new = False
            self.update_screen()
            self.next = ShowDialog("donotpress", self.screen)
            self.pause()
            self.quit()

    def start_level(self, clan, level):
        if clan in player_data["unlocked"] or self.cur_blending or self.cur_blinking:
            #do not start finished maps and wait while blending new signet in
            return
        self.success = False
        self.cur_clan = clan
        self.next = Level(level, prev=self)
        self.pause()
        self.quit()

    def unpause(self):
        if self.success:
            if self.cur_clan:
                self.show_signet([self.cur_clan])
                player_data["unlocked"].append(self.cur_clan) #track progress
            self.cur_clan = None
            self.autosave() #save progress
        State.unpause(self)

    def autosave(self):
        try:
            save_object(player_data, "save.dat")
        except:
            print "Autosave failed. Uh.. guess you need to keep playing?"

    def show_signet(self, clans):
        #show recently gained signet(s)
        self.cur_blending = []
        self.cur_blinking = []
        for clan in clans:
            #signet
            signet = self.signets[clan][0]
            blend = StaticWidget(signet.get_size())
            blend.fill((255,0,255))
            blend.set_colorkey((255,0,255))
            blend.blit(signet, (0,0))
            blend.set_pos(self.signet_pos[clan])
            blend.set_alpha(0)
            self.cur_blending.append(blend)

            #green light
            img = self.light
            blink = StaticWidget(img.get_size())
            blink.fill((255,0,255))
            blink.set_colorkey((255,0,255))
            blink.blit(img, (0,0))
            blink.set_pos(self.light_pos[clan])
            self.cur_blinking.append(blink)
            self.blink_timer.set_interval(3)

    def update_other(self):
        if self.cur_blending:
            #fancy alpha blending effect for new signet
            alpha = self.cur_blending[0].get_alpha() + 5

            if alpha > 100:  #stop blending
                for blend in self.cur_blending:
                    blend.set_alpha(None)
                    blend.draw(self.bg_image)
                self.cur_blending = None
            else: #keep blending
                for blend in self.cur_blending:
                    blend.set_alpha(alpha)
                    blend.draw(self.bg_image)

        elif self.cur_blinking:
            if not self.blink_timer.update():
                return

            #fancy blinking light o/
            self.blink_count += 1
            b = self.blink_count

            #simulate irregular flickering
            if b == 1:
                self.blink_timer.set_interval(2)
            elif b == 3:
                self.blink_timer.set_interval(3)
            elif b == 6:
                self.blink_timer.set_interval(1)

            #to draw or not to draw...
            if self.blink_count % 2:
                for blink in self.cur_blinking:
                    blink.draw(self.ui_layer)
            else:
                self.ui_layer.fill((255,0,255))

            #ok, stop flickering.
            if self.blink_count > self.blink_max:
                for blink in self.cur_blinking:
                    blink.draw(self.bg_image)
                self.ui_layer.fill((255,0,255))
                self.blink_count = 0
                self.cur_blinking = None
        else:
            self.check_mouseover()

    def check_mouseover(self):
        #check if mouse hovers signet and show tooltip
        mpos = pygame.mouse.get_pos()
        for b in self.signet_buttons:
            if b.rect.collidepoint(mpos):
                self.tooltip.fill((255,255,255))
                self.tooltip.draw_border((0,0,0))
                self.tooltip.blit(GLOBALS["kanji"]["clan"], (10,3))
                self.tooltip.blit(GLOBALS["kanji"][b.clan], (60,4))
                pos = (mpos[0] + 10, mpos[1] + 20)
                x = (b.rect.centerx - self.tooltip.get_size()[0]/2)
                y = b.rect.top
                pos = (x,y)
                self.tooltip_layer.blit(self.tooltip, pos)
                break

        dnp = self.donotpress
        if dnp.collidepoint(mpos):
            pos = (dnp.left-60, dnp.top - 22)
            self.tooltip_layer.blit(self.dnp_tooltip, pos)

    def bigred_pressed(self):
        if self.cur_blending or self.cur_blinking:
            return

        if len(player_data["unlocked"]) < 9:
            self.next = ShowDialog("nothinghappened", self.screen)
            self.pause()
            self.quit()
        else:
            self.next = Outro()
            self.quit()

    def update_screen(self):
        self.background.blit(self.bg_image, (0,0))
        self.background.blit(self.tooltip_layer, (0,0))
        self.background.blit(self.ui_layer, (0,0))
        self.tooltip_layer.fill((255,0,255))
        State.update_screen(self)

