import pygame

from pykitten.data import load_object, save_object, filepath

from globs import GLOBALS

#map dict:
# "tset" = tileset (without the .png)
# "tilemap" = keys => coords, values => tiletype
# "objects" = keys => coords, values => [objectname, params]

map_folder = "maps"

tsize = GLOBALS["tile_size"]

class Trigger(pygame.Rect):
    def __init__(self, coord, name, params):
        size = (tsize, tsize*5)
        pos = (coord[0]*tsize - tsize, coord[1]*tsize - size[1]) #make trigger higher
        pygame.Rect.__init__(self, [pos, size])
        self.name = name
        self.params = params
        self.dead = False
        #print "created trigger:", name, params

class ExitTrigger(pygame.Rect):
    def __init__(self, coord, name, params):
        size = (tsize*6, tsize*5)
        pos = (coord[0]*tsize - tsize, coord[1]*tsize - size[1]) #make trigger higher and wider
        pygame.Rect.__init__(self, [pos, size])
        self.name = name
        self.params = params
        self.dead = False

class Map(dict):
    def __init__(self, filename=None):
        dict.__init__(self)

        self["tset"] = "tileset"
        self["tilemap"] = {}
        self["objects"] = {}
        self["max"] = (0,0)
        self["options"] = {}
        self["options"]["clan"] = "fire"

        self["start"] = (128,128)
        self["exit"] = (0,0)
        self.filename = filename
        if self.filename is None:
            self.filename = ""

        try:
            data = load_object(filepath(map_folder,filename))
        except:
            print "Could not load map. Using empty map"
            return

        self["tset"] = data["tset"]
        self["tilemap"] = data["tilemap"]
        self["objects"] = data["objects"]
        self["options"] = data["options"]

    def set_max(self):
        x, y = 0, 0
        for k in self["tilemap"].keys():
            if k[0] > x:
                x = k[0]
            if k[1] > y:
                y = k[1]

        self["max"] = (x+1, y+1)
        return self["max"]

    def draw(self, editing=True):
        def check(coord, values=[None, -1, -1, -1]):
            #None => nothing, -1 => anything
            x, y = coord
            adj = [(x,y-1), (x,y+1), (x-1,y), (x+1,y)]  #up, down, left, right
            for i in xrange(4):
                val = values[i]
                if val is None:
                    if adj[i] in lvl_map.keys(): #None
                        return False
                elif val != -1:
                    if (adj[i] not in lvl_map.keys() or \
                        lvl_map[adj[i]] != val): #specific value
                        return False
            return True

        lvl_map = self["tilemap"]

        tsize = GLOBALS["tile_size"]
        tset = GLOBALS["ts_"+self["tset"]]

        sx, sy = self.set_max()
        if editing:
            surface = pygame.Surface((sx*tsize, sy*tsize))
        else:
            surface = pygame.Surface((sx*tsize, (sy+10)*tsize))

        surface.fill((255,0,255))
        surface.set_colorkey((255,0,255))

        #draw tiles
        for k in self["tilemap"].keys():
            x, y = k
            tiletype = lvl_map[k]
            if tiletype in tset.keys():
                img = tset[tiletype]
                if tiletype == (0,0) and check((x,y)):
                    img = tset[1,0]  #grass on dirt
                pos = (x*tsize, y*tsize)
                surface.blit(img, pos)

        #draw objects, get triggers
        show = []  #show these in-game
        self.enemies = [] #enemy start coords
        self.triggers = [] #rects of triggers
        for k in self["objects"].keys():
            name, params = self["objects"][k]
            x, y = k
            if GLOBALS["objects"].has_key(name):
                if name=="start" or name=="exit":
                    self[name] = k
                if name=="enemy":
                    self.enemies.append(k)
                elif name=="dtrigger":
                    trig = Trigger((x,y), name=name, params=params)
                    self.triggers.append(trig)
                elif name=="exit":
                    trig = ExitTrigger((x,y), name=name, params=params)
                    self.triggers.append(trig)

                if name=="exit" and not self.filename.endswith("tutorial.map"):
                    img = GLOBALS["objects"]["dojo"].copy() #draw dojo
                    draw_x = x * tsize
                    draw_y = y * tsize - 364 + 32
                    #position of black transparend thingy for entry
                    self.dojo_pos = (draw_x + 230, draw_y+275)
                    #blit clan signets on dojo
                    clan = self["options"]["clan"]
                    if GLOBALS["signets"].has_key(clan):
                        signets = GLOBALS["signets"][clan]
                    else:
                        signets = GLOBALS["signets"]["fire"]
                    medium = signets[1]
                    tiny = signets[2]
                    img.blit(medium, (235,29))
                    img.blit(tiny, (242,240))
                    surface.blit(img, (draw_x, draw_y)) #draw on map

                elif name=="exit" and self.filename.endswith("tutorial.map"):
                    #draw mysterious
                    img = GLOBALS["ui_stuff"]["mysterious.png"].copy()
                    draw_x = x * tsize + 3*tsize
                    draw_y = y * tsize - 227 + 32
                    self.dojo_pos = (800,600) #place black transparent dojo thingy off screen
                    #blit closed doors
                    doors = GLOBALS["ui_stuff"]["door_both.png"]
                    img.blit(doors, (99,63))
                    surface.blit(img, (draw_x, draw_y))


                if editing or name in show:
                    img = GLOBALS["objects"][name]
                    pos = (x*tsize, y*tsize)
                    surface.blit(img, pos)

        #add earth under map bottom
        earth = tset[0,0]
        y = self["max"][1] - 1
        for x in xrange(self["max"][0]):
            if lvl_map.has_key((x,y)) and lvl_map[x,y] in [(0,0),(1,0)]:
                for add in xrange(1, 6):
                    pos = (x*tsize, (y+add)*tsize)
                    surface.blit(earth, pos)


        #get horizontal blocks
        blocks = []
        blocking = [(0,0), (1,0), (2,0)]
        for y in xrange(self["max"][1]+1):
            pos_x, pos_y = None, None
            width, height = 0, 1
            for x in xrange(self["max"][0]+1):
                if (x,y) in lvl_map.keys():# and lvl_map[x,y] in blocking:
                    if pos_x is None:
                        pos_x = x
                        pos_y = y
                    width += 1
                elif pos_x is not None:
                    b = pygame.Rect([pos_x*tsize, pos_y*tsize, width*tsize, tsize])
                    blocks.append(b)
                    pos_x, pos_y, width, height = None, None, 0, 1

        #add invisible blocks around map boundaries
        vsize = (32, self["max"][1]*32)
        b = pygame.Rect([(-32, 0), vsize])
        blocks.append(b)
        b = pygame.Rect([(self["max"][0]*32-32, 0), vsize])
        blocks.append(b)



        return surface, blocks

    def save_map(self, filename="tmp", path=""):
        if not path:
            path = filepath(map_folder, filename)
        mapdata = {}
        keys = ["tilemap", "tset", "objects", "options"]
        for k in keys:
            mapdata[k] = self[k]
        save_object(mapdata, path)



