import pygame

from pykitten.utils import Timer
from pykitten.kittens import Kitten, AnimatedKitten

from globs import GLOBALS




sfx_jump = "jump2.wav"
sfx_land = "land2.wav"
sfx_throw = "throw2.wav"
sfx_die = "land4.wav"


player_data = GLOBALS["player_data"]

class Ninja(AnimatedKitten):
    def __init__(self, cam, images):
        self.all_images = images
        self.camera = cam
        AnimatedKitten.__init__(self, self.all_images["idle"], delay=10)
        self.colrect = pygame.Rect([0,0,35,35])
        self.angle = 0  #rotation angle for death animation
        self.shuriken = None

        self.idle_timer = Timer(200)
        self.jump_timer = Timer(20)
        self.shuriken_timer = Timer(20)
        self.invincible = 0
        self.invincible_timer = Timer(100)  #on hit nanji is invincible for set time
        self.blink_timer = Timer(2)

        self.name = ""
        self.cur_images = "idle"
        self.speed = 4
        self.look = 1 #1 => looking right

        self.statlist = [
            "idle",  #0
            "walking",  #1
            "sprinting",  #2
            "throwing",  #3
            "jumping",  #4
            "falling",  #5
            "landing",  #6
            "ducking", #7
            "dying", #8
        ]

        self.on_ground = [0,1,2]
        self.status = 0
        self.fall()

    def set_image(self, img):
        #return  #DELETEME
        top = self.rect.top
        bottom = self.rect.bottom
        left = self.rect.left
        right = self.rect.right

        vx, vy = self.vel

        AnimatedKitten.set_image(self, img)
        self.rect.size = self.image.get_size()

        if vy < 0:
            self.rect.top = top
        else:
            self.rect.bottom = bottom

        if self.look > 0:
            self.rect.left = left
        else:
            self.rect.right = right

        self.rect.centerx = self.colrect.centerx
        self.rect.bottom = self.colrect.bottom
        self.set_pos(self.rect.topleft)

    def get_hit(self):
        if not self.invincible:
            self.invincible = 1
            self.invincible_timer.activate()

    def bounce(self):
        #TODO: bouncing looks.. kind of not bouncy
        self.status = 4
        self.jump_timer.activate()
        self.jump_timer.counter = self.jump_timer.interval -2
        #GLOBALS["sfx"][sfx_bounce].play()

    def die(self):
        if self.status == 3 and self.shuriken:
            self.shuriken.kill() #remove shuriken
        self.status = 8
        GLOBALS["sfx"][sfx_die].play()

    def change_images(self, name, lookupdate=False):
        change = self.cur_images != name
        if change or lookupdate:
            nname = name
            nr = self.cur_image
            counter = self.anim_timer.counter

            if self.look < 0:
                nname = name + "_l"

            AnimatedKitten.set_images(self, self.all_images[nname])

            self.cur_images = name

            if name=="sprinting"or name=="throwing":
                self.anim_timer.set_interval(3)
            elif name=="jump":
                self.anim_timer.set_interval(5)
            elif name=="landing":
                self.anim_timer.set_interval(7)
            else:
                self.anim_timer.set_interval(10)

            if lookupdate:
                self.set_image(self.images[nr])
                self.cur_image = nr
                self.anim_timer.counter = counter



            #self.stop()

    def sprint(self):
        if self.status in self.on_ground:
            self.status = 2

    def unsprint(self):
        if self.status == 2:
            self.status = 0

    def jump(self):
        if self.status in self.on_ground:
            GLOBALS["sfx"][sfx_jump].play()
            self.status = 4
            self.jump_timer.activate()

    def unjump(self):
        if self.status == 4 and self.jump_timer.counter > 8:
            self.status = 5
            self.jump_timer.deactivate()

    def land(self):
        if self.status == 5:
            self.status = 6
            GLOBALS["sfx"][sfx_land].play()

    def fall(self):
        self.status = 5

    def throw(self):
        if (self.status in self.on_ground or self.status==7) and \
            self.status != 3 and not self.shuriken_timer.active:
            self.status = 3
            GLOBALS["sfx"][sfx_throw].play()
            self.mod_vel(vx=0,vy=0) #stand still while throwing!!
            self.change_images("idle")
            self.shuriken_timer.activate()
            return True
        return False

    def unthrow(self):
        if self.status == 3:
            self.status = 0

    def turn(self):
        self.look = self.look * - 1
        self.change_images(self.cur_images, lookupdate=True)

    def duck(self):
        if self.status in self.on_ground:
            self.status = 7

    def unduck(self):
        self.status = 0

    def mod_vel(self, vx=None, vy=None):
        nx, ny = self.vel
        if vx is not None:
            nx = vx
        elif nx > 0:
            nx = 1
        elif nx < 0:
            nx = -1

        if vy is not None:
            ny = vy
        elif ny > 0:
            ny = 1
        elif ny < 0:
            ny = -1

        self.set_vel((nx,ny))

    def set_vel(self, vel):
        vx, vy = vel
        sprint, jump = 0, 0
        if self.status==2:
            sprint = 6 * vx
        if self.status==4 and vy < 0:
            jump = 4
        AnimatedKitten.set_vel(self, (vx*self.speed+sprint, vy*self.speed-jump))

    def update_status(self):
        if self.vel[0] > 0 and self.look < 0:
            self.turn()
        elif self.vel[0] < 0 and self.look > 0:
            self.turn()

        if self.status == 8: #dying :/
            self.image = pygame.transform.rotate(self.all_images["jump"][2], self.angle)
            self.angle += 5
            self.pause()
            return

        if self.status in self.on_ground:
            if self.status==2 and self.vel[0]:
                self.change_images("sprinting")

            elif self.vel[0]:
                self.status = 1
                self.change_images("walking")

            elif not self.cur_images=="idle":
                self.change_images("idle")
                self.status = 0
                self.idle_timer.activate()
                self.stop()

        elif self.status==7:
            self.set_vel((0,0))
            self.change_images("ducking")
        elif self.status==3:
            self.change_images("throwing")
            self.mod_vel(vx=0)
            if self.cur_image >= len(self.images) -1 and \
                self.anim_timer.counter >= self.anim_timer.interval -1:
                self.unthrow()
        elif self.status==6:
            self.change_images("landing")
            if self.cur_image >= len(self.images) -1:
                self.status = 0
                self.change_images("idle")
            elif self.cur_image == 1 and self.anim_timer.interval < 10:
                self.anim_timer.set_interval(10)

        elif self.status==5:
            self.mod_vel(vy=1)
            self.change_images("flight")

        elif self.status==4:
            self.change_images("jump")

        if self.status in [4,5] and self.cur_image >= len(self.images) -1:
            self.pause()  #pause last image of falling and jumping
        elif self.status==0:
            if self.idle_timer.update():  #play idle animation after some time
                self.play()
                self.idle_timer.deactivate()
        else:
            self.play()

        if self.shuriken_timer.update():
            self.shuriken_timer.deactivate()

    def update(self):
        if self.status==4:
            if self.cur_image < 3:
                self.mod_vel(vy=0)
            else:
                self.mod_vel(vy=-1)
                if self.jump_timer.update():
                    self.jump_timer.counter = 9
                    self.unjump()

        #cannibalized from AnimatedKitten.update because that code was a booboo
        if self.animate and self.anim_timer.update():
            self.cur_image += 1
            if self.cur_image >= len(self.images):
                    self.cur_image = 0
            self.set_image(self.images[self.cur_image])

        if self.invincible:
            if self.blink_timer.update():
                if self.image.get_alpha() > 0:
                    self.image = self.image.copy()
                    self.image.set_alpha(0)
                else:
                    self.image = self.image.copy()
                    self.image.set_alpha(255)
            if self.invincible_timer.update():
                self.image = self.image.copy()
                self.image.set_alpha(255)
                self.invincible_timer.deactivate()
                self.invincible = 0

    def set_pos(self, pos):
        AnimatedKitten.set_pos(self, pos)
        self.colrect.centerx = self.rect.centerx
        self.colrect.bottom = self.rect.bottom

    def check_mapcollisions(self, blocks):
        bl = blocks
        vx, vy = self.vel
        px, py = self.colrect.topleft
        width, height = self.colrect.size
        test = 0
        if vy == 0:
            vy = 1
            test = 1
        colliding = [0, 0]

        #vertical collisions
        vrect = pygame.Rect([px, py+vy, width, height])
        blocks = vrect.collidelistall(bl)
        for nr in blocks:
            block = bl[nr]
            colliding[1] = vy
            if vy > 0 and vrect.bottom > block.top and not self.status==6:
                if not test:
                    self.land()
                self.colrect.bottom = block.top
                self.mod_vel(vy=0)
            elif vy < 0 and vrect.top < block.bottom:
                self.unjump()
                self.colrect.top = block.bottom
                self.mod_vel(vy=0)

        if test and not blocks and not self.status==4 and not self.status==6:
            #fall unless you are jumping or landing
            self.fall()
        elif not blocks:
            self.colrect.top += vy #kk, move along

        #horizontal collisions
        hrect = pygame.Rect([px+vx, py, width, height])
        blocks = hrect.collidelistall(bl)
        for nr in blocks:
            block = bl[nr]
            colliding[0] = vx
            if vx > 0 and hrect.right > block.left:
                self.colrect.right = block.left
                self.mod_vel(vx=0)
            elif vx < 0 and hrect.left < block.right:
                self.colrect.left = block.right
                self.mod_vel(vx=0)
        if not blocks:
            self.colrect.left += vx #gogo little kitteh!

        self.rect.midbottom = self.colrect.midbottom
        self.set_pos(self.rect.topleft)

        return colliding

    def off_screen(self):
        #returns true if character off screen
        off_x, off_y = self.camera.pos
        real_x, real_y = self.pos[0] - off_x, self.pos[1] - off_y
        screen_x, screen_y = GLOBALS["screen_size"]
        w, h = self.get_size()
        #set larger screen to keep updating ninjats
        w += 600
        h += 100
        off_x -= 300
        off_y -= 50
        return real_x > screen_x+w or real_y > screen_y+h or real_x < -w or real_y < -h


    def check_trigcollisions(self, trigger):
        pass #keep this.

class Nanji(Ninja):
    def __init__(self, cam):
        Ninja.__init__(self, cam, GLOBALS["nanji"])
        self.name = "nanji"
        self.ai = 0
        self.exiting = 0

    def check_trigcollisions(self, triggers):
        nr = self.colrect.collidelist(triggers)
        if nr == -1:
            return
        trig = triggers[nr]  #pop to remove trigger
        if trig.dead:
            return

        trig.dead = True
        #print trig.name, trig.params
        return trig

    def update(self):
        if self.status==8: #not visible or dying
            px, py = self.pos
            self.set_pos((px, py+4))
            return
        Ninja.update(self)

    def off_screen(self):
        #returns true if character off screen
        off_x, off_y = self.camera.pos
        real_x, real_y = self.pos[0] - off_x, self.pos[1] - off_y
        screen_x, screen_y = GLOBALS["screen_size"]
        return real_y >= screen_y

class Ninjat(Ninja):
    def __init__(self, cam, clan="fire"):
        if not GLOBALS["ninjats"].has_key(clan):
            clan = "fire"
        Ninja.__init__(self, cam, GLOBALS["ninjats"][clan])
        self.name = "ninjat"
        self.ai = 1
        self.speed = 2
        self.set_vel((-1,0))
        self.sit_timer = Timer(10)
        self.shuriken_timer.set_interval(70)
        self.reaction_timer = Timer(10)


    def update(self):
        Ninja.update(self)

    def ai_update(self, parent):
        if self.off_screen():
            return 0
        elif self.status==8: #not visible or dying
            px, py = self.pos
            self.set_pos((px, py+4))
            return 0

        blocks = parent.blocks
        vx, vy = self.vel

        #can you see nanji?
        nanji = parent.player
        if not nanji.status==7 and not self.shuriken_timer.active:
            nanji_rect = nanji.colrect
            own_rect = self.colrect
            in_height = own_rect.top <= nanji_rect.centery <= own_rect.bottom
            if self.look < 0:
                in_look = own_rect.left > nanji_rect.left
            else:
                in_look = own_rect.right < nanji_rect.right

            if in_height and in_look and self.reaction_timer.update():
                if not self.shuriken_timer.active:
                    self.throw()
                    self.update()
                    self.reaction_timer.activate()
                    return 1

        #loockityerfeets, little kitteh - try not to fall down >.<
        if self.look > 0:
            x = self.colrect.right + vx
            x2 = self.colrect.left
        else:
            x = self.colrect.left - vx
            x2 = self.colrect.right
        front_paws = pygame.Rect([x, self.colrect.bottom+10, 2, 2])
        back_paws = pygame.Rect([x2, self.colrect.bottom+10, 2, 2])

        if front_paws.collidelist(blocks) < 0: #no graunds in front :/
            if back_paws.collidelist(blocks) > -1: #but grawnds in bak. go bak! :)
                self.turn()
                self.mod_vel(vx=self.look)
            else:
                self.mod_vel(vx=0) #fall down.


        #turn around if bumped against block
        col_x, col_y = self.check_mapcollisions(blocks)
        if col_x:
            self.turn()
            self.mod_vel(vx=self.look)

        self.update()
        if self.status == 0 and self.sit_timer.update():
            self.set_vel((self.look,0))
        return 0

