import pygame
from pygame.locals import *

from pykitten.utils import Timer
from pykitten.engine import State
from pykitten.kittens import Kitten
from pykitten.gui import StaticWidget


from globs import GLOBALS
from nanji import Nanji

class Vortex(Kitten):
    def __init__(self, centerpos):
        img = GLOBALS["ui_stuff"]["vortex.png"]
        Kitten.__init__(self, img)
        self.angle = 0
        self.angle_add = 10
        self.image_copy = img.copy()
        self.centerpos = centerpos
        self.rect.centerx = centerpos[0]
        self.rect.top = centerpos[1]
        self.set_pos(self.rect.topleft)
        self.center_diff = 1
        self.center_diff_timer = Timer(5)

    def update(self):
        center = self.rect.center

        self.angle += self.angle_add
        if self.angle > 360:
            self.angle = self.angle - 360
        self.image = pygame.transform.rotate(self.image_copy, self.angle)
        self.rect.size = self.image.get_size()
        self.rect.center = center

        if self.center_diff_timer.update():
            #small movement around center
            self.rect.centerx = center[0] + self.center_diff
            self.rect.centery = center[1] + self.center_diff
            self.center_diff *= -1
        self.set_pos(self.rect.topleft)
        Kitten.update(self)

class Outro(State):
    def __init__(self):
        State.__init__(self)
        self.fps = 40

        #animation checkpoints and flags
        self.opening = 0
        self.closing = 0
        self.door_timer = Timer(15) #open slowly, then faster
        self.maelstrom_timer = Timer(30) #open doors a bit.. then pull
        self.nanji_command = None

        #generate pseudo-map
        screen_size = GLOBALS["screen_size"]
        self.bg_image = pygame.Surface(screen_size)
        self.top_layer = pygame.Surface(screen_size)
        self.top_layer.set_colorkey((255,0,255))
        self.top_layer.fill((255,0,255))

        #draw map background
        bg = GLOBALS["backgrounds"]["fuji.png"]
        self.bg_image.blit(bg, (0,0))

        #draw grass
        tsize = GLOBALS["tile_size"]
        grass_tile = GLOBALS["ts_tileset"][1,0]
        x = -tsize
        y = screen_size[1] - tsize
        for i in xrange(22):
            self.top_layer.blit(grass_tile, (x,y))
            x += tsize

        #draw mysterious
        myst = GLOBALS["ui_stuff"]["mysterious.png"]

        #draw full signet board on mysterious
        board = GLOBALS["ui_stuff"]["board_filled.png"]
        x, y = 66, 197
        myst.blit(board, (x,y))

        x = 5 * tsize
        y = screen_size[1] - myst.get_size()[1] - tsize
        self.myst = StaticWidget(myst.get_size())
        self.myst.set_colorkey((255,0,255))
        self.myst.fill((255,0,255))
        self.myst.blit(myst, (0,0))
        self.myst.set_pos((x,y))
        self.myst.pos = (x,y)
        self.myst.draw(self.top_layer)
        myst_centerx = x + myst.get_size()[0]/2
        myst_top = y + 30

        #init doors
        self.door_speed = 1
        door_left_image = GLOBALS["ui_stuff"]["door_left.png"]
        door_right_image = GLOBALS["ui_stuff"]["door_right.png"]
        self.door_left = StaticWidget(door_left_image.get_size())
        self.door_left.blit(door_left_image, (0,0))
        self.door_left.set_pos((self.myst.pos[0]+99,self.myst.pos[1]+63))
        self.door_right = StaticWidget(door_right_image.get_size())
        self.door_right.blit(door_right_image, (0,0))
        self.door_right.set_pos((self.myst.pos[0]+177 , self.myst.pos[1]+63))

        #init whirling vortex
        self.vortex = Vortex(centerpos=(myst_centerx, myst_top))

        #init Nanji
        self.nanji = Nanji(cam=None)
        self.nanji.status = 0
        self.nanji.set_pos((205,401))
        self.nanji_angle = 0
        self.nanji_fly_img = self.nanji.all_images["landing"][0]
        self.nanji_shrink = 1
        self.sprites.add(self.nanji)

        self.nanji_do("throw")

    def main_start(self):
        pygame.mouse.set_visible(0)
        State.main_start(self)

    def update_doors(self):
        if self.opening:
            lx, ly = self.door_left.get_pos()
            rx, ry = self.door_right.get_pos()
            self.door_left.set_pos((lx-self.door_speed, ly))
            self.door_right.set_pos((rx+self.door_speed, ry))
            if self.door_timer.update():
                self.door_speed = 2
            self.opening = lx > 165
        elif self.closing:
            self.door_speed = 1
            lx, ly = self.door_left.get_pos()
            rx, ry = self.door_right.get_pos()
            self.door_left.set_pos((lx+1, ly))
            self.door_right.set_pos((rx-1, ry))
            self.closing = lx < 258
            if not self.closing:
                self.maelstrom_timer.set_interval(50)
                self.nanji_do("waitquit")

        self.door_left.draw(self.background)
        self.door_right.draw(self.background)

    def nanji_do(self, do="throw"):
        self.nanji_command = do

        if do=="throw": #lift paw
            self.nanji.change_images("throwing")
            self.nanji.anim_timer.set_interval(5)
            self.nanji.play()
        elif do=="unthrow":  #put paw down
            images = self.nanji.images[:]
            images.reverse()
            self.nanji.set_images(images)
            self.nanji.play()
        elif do=="wait_door":
            self.nanji.change_images("idle")
            self.opening = 1
            self.nanji.pause()
        elif do=="walk" or do=="walk2":
            self.nanji.look = -1
            self.nanji.change_images("walking")
            self.nanji.anim_timer.set_interval(8)
            self.nanji.play()
        elif do=="sprint" or do=="sprint2":
            self.nanji.change_images("sprinting")
            self.nanji.anim_timer.set_interval(8)
            self.nanji.play()
        elif do=="duck":
            self.nanji.change_images("ducking")
            self.nanji.pause()
        elif do=="lift":
            self.nanji.change_images("landing")
            self.nanji.pause()

    def update_nanji(self):
        if self.nanji_command is None:
            return

        #update old command
        cmd = self.nanji_command
        x, y = self.nanji.pos #get pulled
        if cmd=="throw" and self.nanji.cur_image >= 3: #paw up
            self.nanji_do("unthrow")
        elif cmd=="unthrow" and self.nanji.cur_image >= 3: #paw down
            self.nanji_do("wait_door")
        elif cmd=="wait_door" and self.maelstrom_timer.update(): #wait
            self.nanji_do("walk")
        elif cmd=="walk":
            if x > 150: #walk left
                self.nanji.set_pos((x-1, y))
            else:
                self.nanji_do("walk2")
        elif cmd=="walk2":
            if x < 190:
                self.nanji.set_pos((x+1, y))
            else:
                self.nanji_do("sprint")
        elif cmd=="sprint":  #try to sprint
            if x > 170:
                self.nanji.set_pos((x-1, y))
            else:
                self.nanji_do("sprint2")
        elif cmd=="sprint2":
            self.nanji.set_pos((x+1, y))
            if self.maelstrom_timer.update():
                self.nanji_do("duck")
        elif cmd=="duck" and self.maelstrom_timer.update():
            self.nanji_do("duckpull")
        elif cmd=="duckpull":
            if x < 250:
                self.nanji.set_pos((x+1, y))
            else:
                self.nanji_do("lift")
        elif cmd=="lift":
            if x < 280:
                self.nanji.set_pos((x+1, y))
            else:
                self.nanji_do("liftoff")
                self.maelstrom_timer.set_interval(10)
        elif cmd=="liftoff":
            self.nanji.set_pos((x, y-1))
            if self.maelstrom_timer.update():
                self.nanji_do("whirl")
        elif cmd=="whirl":
            img = self.nanji_fly_img.copy()
            sx, sy = img.get_size()
            img = pygame.transform.scale(img, (sx-self.nanji_shrink, sy-self.nanji_shrink))
            self.nanji_shrink += 1
            if sx-self.nanji_shrink < 0 or sy-self.nanji_shrink < 0:
                self.nanji_do("done")

            self.nanji_angle += 10
            if self.nanji_angle > 360:
                self.nanji_angle = self.nanji_angle - 360
            img = pygame.transform.rotate(img, self.nanji_angle)

            center = self.nanji.rect.center
            self.nanji.set_image(img)
            self.nanji.rect.center = center

            if y > 350:
                y -= 1
            if x < 350:
                x += 1
            self.nanji.set_pos((x,y))
        elif cmd=="done":
            self.nanji_do(None)
            self.closing = 1
        elif cmd=="waitquit" and self.maelstrom_timer.update():
            self.next = TheEnd()
            self.quit()

    def update_screen(self):
        self.background.blit(self.bg_image, (0,0))

        #update and draw vortex
        self.vortex.update()
        self.background.blit(self.vortex.image, self.vortex.pos)

        #update and draw doors
        self.update_doors()

        #draw grass, building
        self.background.blit(self.top_layer, (0,0))

        #draw nanji
        self.update_nanji()
        State.update_screen(self)

class TheEnd(State):
    def __init__(self):
        State.__init__(self)
        self.bg_image = GLOBALS["backgrounds"]["the_end.png"]
        self.overlay = StaticWidget((638,169))
        self.overlay.set_pos((2,160))
        self.overlay.set_alpha(255)
        self.overlay.fill((0,0,0))
        self.wait = Timer(50)

    def main_start(self):
        pygame.mouse.set_visible(0)
        GLOBALS["jb"].stop()

    def update_screen(self):
        self.background.fill((0,0,0))
        self.background.blit(self.bg_image, (0,0))

        if not self.wait.active:
            alpha = self.overlay.get_alpha()
            if alpha > 0:
                self.overlay.set_alpha(alpha-10)
            if alpha < 20:
                self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)
        elif self.wait.update():
            self.wait.deactivate()

        self.overlay.draw(self.background)
        State.update_screen(self)

