import pygame
from pygame.locals import *

from pykitten import utils
from pykitten.data import load_object
from pykitten.gui import TextLine, ImgButton, ClickArea
from pykitten.engine import State

from level import Level
from levelselect import LevelSelection
from globs import GLOBALS, init_player_data
from fadestate import FadeState


class StartMenu(State):
    def __init__(self):
        State.__init__(self)

        self.background.blit(GLOBALS["backgrounds"]["main_menu.png"], (0,0))

        buttons = {
            " Start ": pygame.Rect([53,228,132,119]),
            " Resume ": pygame.Rect([246,184,151,117]),
            " Exit ": pygame.Rect([458,232,126,114]),
            " Credits ": pygame.Rect([285,398,69,64])
        }

        for b in buttons.keys():
            ca = ClickArea(buttons[b], self.button_pressed, name=b)
            self.widgets.append(ca)

    def main_start(self):
        pygame.mouse.set_visible(1)

    def button_pressed(self, name):
        if name==" Start ":
            init_player_data()
            self.next = LevelSelection(new=True)
            self.pause()
            self.quit()
        elif name == " Resume ":
            #lives, health, unlocked
            try:
                pdata = load_object("save.dat")
            except:
                print "Could not load savegame."
            if not pdata:
                return
            lives = pdata["lives"]
            health = pdata["health"]
            unlocked = pdata["unlocked"]
            init_player_data(lives, health, unlocked)
            self.next = LevelSelection(new=False)
            self.pause()
            self.quit()
        elif name == " Credits ":
            self.next = Credits()
            self.pause()
            self.quit()

        elif name== " Exit ":
            print "Thanks for playing! :)"
            self.quit()


class Credits(State):
    def __init__(self):
        State.__init__(self)
        self.background.blit(GLOBALS["backgrounds"]["credits.png"], (0,0))
        ca = ClickArea(pygame.Rect((0,0),GLOBALS["screen_size"]), self.quit)
        self.widgets.append(ca)
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)


class SplashScreen(State):
    def __init__(self):
        State.__init__(self)
        img = GLOBALS["ui_stuff"]["logo.png"]
        self.background.blit(img,
            utils.center(img.get_size(), GLOBALS["screen_size"]))
        self.quit_timer = utils.Timer(50)
        self.quit_timer.deactivate()
        GLOBALS["sfx"]["pululummm.wav"].play()

    def main_start(self):
        if not self.quit_timer.active:

            self.quit_timer.activate()

    def update_other(self):
        if self.quit_timer.update():
            next = StartMenu()
            self.next = FadeState(self.background, next.background, next)
            self.quit()
